# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing_extensions
import typing
from ..core.serialization import FieldMetadata
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomerMetadataUpdateParams(UniversalBaseModel):
    """
    Parameters for updating a customer's extra metadata.
    """

    set_: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Optional[str]]],
        FieldMetadata(alias="set"),
    ] = pydantic.Field(default=None)
    """
    Key-value pairs (`str: str`) to describe the customer
    """

    unset: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Extra metadata keys to remove
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Capability(UniversalBaseModel):
    """
    A capability represents a functional feature or skill of the agent.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the capability
    """

    title: str = pydantic.Field()
    """
    The title of the capability
    """

    description: str = pydantic.Field()
    """
    Detailed description of the capability's purpose
    """

    signals: typing.List[str] = pydantic.Field()
    """
    Example signals that this capability can handle
    """

    tags: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of tag IDs associated with the capability
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .guideline_content import GuidelineContent
import typing
from .tool_id import ToolId
from .guideline_payload_operation_dto import GuidelinePayloadOperationDto
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class GuidelinePayload(UniversalBaseModel):
    """
    Payload data for a Guideline operation
    """

    content: GuidelineContent
    tool_ids: typing.List[ToolId]
    operation: GuidelinePayloadOperationDto
    updated_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the guideline
    """

    action_proposition: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the action proposition is enabled
    """

    properties_proposition: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Properties proposition
    """

    journey_node_proposition: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Journey step proposition
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .criticality_dto import CriticalityDto
from .composition_mode_dto import CompositionModeDto
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Guideline(UniversalBaseModel):
    """
    Represents a guideline.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the guideline
    """

    condition: str = pydantic.Field()
    """
    If this condition is satisfied, the action will be performed
    """

    action: typing.Optional[str] = pydantic.Field(default=None)
    """
    This action will be performed if the condition is satisfied
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional description providing additional context for the guideline
    """

    criticality: typing.Optional[CriticalityDto] = pydantic.Field(default=None)
    """
    The criticality level of the guideline
    """

    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the guideline is enabled
    """

    tags: typing.List[str] = pydantic.Field()
    """
    The tags associated with the guideline
    """

    metadata: typing.Dict[str, typing.Optional[typing.Any]] = pydantic.Field()
    """
    Metadata for the guideline
    """

    composition_mode: typing.Optional[CompositionModeDto] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .agent_message_guideline_rationale_dto import AgentMessageGuidelineRationaleDto
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class AgentMessageGuideline(UniversalBaseModel):
    action: str = pydantic.Field()
    """
    A single action that explains what to say; i.e. "Tell the customer that you are thinking and will be right back with an answer."
    """

    rationale: typing.Optional[AgentMessageGuidelineRationaleDto] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .composition_mode_dto import CompositionModeDto
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Journey(UniversalBaseModel):
    """
    A journey represents a guided interaction path for specific user scenarios.

    Each journey is triggered by a condition and contains steps to guide the interaction.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the journey
    """

    title: str = pydantic.Field()
    """
    The title of the journey
    """

    description: str
    conditions: typing.List[str]
    tags: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of tag IDs associated with the journey
    """

    composition_mode: typing.Optional[CompositionModeDto] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

CompositionModeDto = typing.Union[
    typing.Literal["fluid", "canned_fluid", "composited_canned", "strict_canned"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

AgentMessageGuidelineRationaleDto = typing.Union[
    typing.Literal["unspecified", "buy_time", "follow_up"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ...types.session_listing import SessionListing
from ...types.session import Session

SessionsListResponse = typing.Union[SessionListing, typing.List[Session]]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
from .event_source_dto import EventSourceDto
from .event_kind_dto import EventKindDto
import datetime as dt
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Event(UniversalBaseModel):
    """
    Represents a single event within a session.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the event
    """

    source: EventSourceDto
    kind: EventKindDto
    offset: int = pydantic.Field()
    """
    Sequential position of the event in the session
    """

    creation_utc: dt.datetime = pydantic.Field()
    """
    UTC timestamp of when the event was created
    """

    trace_id: str = pydantic.Field()
    """
    ID linking related events together
    """

    correlation_id: str = pydantic.Field()
    """
    ID linking related events together
    """

    data: typing.Optional[typing.Any] = None
    metadata: typing.Dict[str, typing.Optional[typing.Any]] = pydantic.Field()
    """
    Metadata associated with the event
    """

    deleted: bool

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

RelationshipKindDto = typing.Union[
    typing.Literal[
        "entailment",
        "priority",
        "dependency",
        "disambiguation",
        "overlap",
        "reevaluation",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ...types.paginated_customers import PaginatedCustomers
from ...types.customer import Customer

CustomersListResponse = typing.Union[PaginatedCustomers, typing.List[Customer]]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class OpenApiServiceParams(UniversalBaseModel):
    """
    Configuration parameters for OpenAPI-based service integration.

    OpenAPI services are integrated using their OpenAPI/Swagger specification,
    enabling automatic generation of client code and documentation.
    """

    url: str = pydantic.Field()
    """
    Base URL for the service. Must include http:// or https:// scheme.
    """

    source: str = pydantic.Field()
    """
    URL or filesystem path to the OpenAPI specification.
            For URLs, must be publicly accessible.
            For filesystem paths, the server must have read permissions.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import datetime as dt
import typing
from .session_mode_dto import SessionModeDto
from .consumption_offsets import ConsumptionOffsets
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Session(UniversalBaseModel):
    """
    A session represents an ongoing conversation between an agent and a customer.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the session
    """

    agent_id: str = pydantic.Field()
    """
    Unique identifier for the agent associated with the session.
    """

    customer_id: str = pydantic.Field()
    """
    ID of the customer associated with this session.
    """

    creation_utc: dt.datetime = pydantic.Field()
    """
    UTC timestamp of when the session was created
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Descriptive title for the session
    """

    mode: SessionModeDto = pydantic.Field()
    """
    The mode of the session, either 'auto' or 'manual'. In manual mode, events added to a session will not be responded to automatically by the agent.
    """

    consumption_offsets: ConsumptionOffsets
    metadata: typing.Dict[str, typing.Optional[typing.Any]] = pydantic.Field()
    """
    Metadata for the session
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..types.relationship_kind_dto import RelationshipKindDto
from ..core.request_options import RequestOptions
from ..types.relationship import Relationship
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.tool_id import ToolId
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.jsonable_encoder import jsonable_encoder
from ..errors.not_found_error import NotFoundError
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RelationshipsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        kind: typing.Optional[RelationshipKindDto] = None,
        indirect: typing.Optional[bool] = None,
        guideline_id: typing.Optional[str] = None,
        tag_id: typing.Optional[str] = None,
        tool_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Relationship]:
        """
        List relationships.

        Either `guideline_id` or `tag_id` or `tool_id` must be provided.

        Parameters
        ----------
        kind : typing.Optional[RelationshipKindDto]

        indirect : typing.Optional[bool]
            Whether to include indirect relationships

        guideline_id : typing.Optional[str]

        tag_id : typing.Optional[str]

        tool_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Relationship]
            Relationships successfully retrieved. Returns a list of all relationships.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.relationships.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "relationships",
            method="GET",
            params={
                "kind": kind,
                "indirect": indirect,
                "guideline_id": guideline_id,
                "tag_id": tag_id,
                "tool_id": tool_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Relationship],
                    parse_obj_as(
                        type_=typing.List[Relationship],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        kind: RelationshipKindDto,
        source_guideline: typing.Optional[str] = OMIT,
        source_tag: typing.Optional[str] = OMIT,
        source_tool: typing.Optional[ToolId] = OMIT,
        target_guideline: typing.Optional[str] = OMIT,
        target_tag: typing.Optional[str] = OMIT,
        target_tool: typing.Optional[ToolId] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Relationship:
        """
        Create a relationship.

        A relationship is a relationship between a guideline and a tag.
        It can be created between a guideline and a tag, or between two guidelines, or between two tags.

        Parameters
        ----------
        kind : RelationshipKindDto

        source_guideline : typing.Optional[str]
            Unique identifier for the guideline

        source_tag : typing.Optional[str]
            Unique identifier for the tag

        source_tool : typing.Optional[ToolId]

        target_guideline : typing.Optional[str]
            Unique identifier for the guideline

        target_tag : typing.Optional[str]
            Unique identifier for the tag

        target_tool : typing.Optional[ToolId]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Relationship
            Relationship successfully created. Returns the created relationship.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.relationships.create(
            source_guideline="gid_123",
            target_tag="tid_456",
            kind="entailment",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "relationships",
            method="POST",
            json={
                "source_guideline": source_guideline,
                "source_tag": source_tag,
                "source_tool": convert_and_respect_annotation_metadata(
                    object_=source_tool, annotation=ToolId, direction="write"
                ),
                "target_guideline": target_guideline,
                "target_tag": target_tag,
                "target_tool": convert_and_respect_annotation_metadata(
                    object_=target_tool, annotation=ToolId, direction="write"
                ),
                "kind": kind,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Relationship,
                    parse_obj_as(
                        type_=Relationship,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve(
        self,
        relationship_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Relationship:
        """
        Read a relationship by ID.

        Parameters
        ----------
        relationship_id : str
            identifier of relationship

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Relationship
            Relationship successfully retrieved. Returns the requested relationship.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.relationships.retrieve(
            relationship_id="gr_123",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"relationships/{jsonable_encoder(relationship_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Relationship,
                    parse_obj_as(
                        type_=Relationship,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        relationship_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete a relationship by ID.

        Parameters
        ----------
        relationship_id : str
            identifier of relationship

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.relationships.delete(
            relationship_id="gr_123",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"relationships/{jsonable_encoder(relationship_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRelationshipsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        kind: typing.Optional[RelationshipKindDto] = None,
        indirect: typing.Optional[bool] = None,
        guideline_id: typing.Optional[str] = None,
        tag_id: typing.Optional[str] = None,
        tool_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Relationship]:
        """
        List relationships.

        Either `guideline_id` or `tag_id` or `tool_id` must be provided.

        Parameters
        ----------
        kind : typing.Optional[RelationshipKindDto]

        indirect : typing.Optional[bool]
            Whether to include indirect relationships

        guideline_id : typing.Optional[str]

        tag_id : typing.Optional[str]

        tool_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Relationship]
            Relationships successfully retrieved. Returns a list of all relationships.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.relationships.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "relationships",
            method="GET",
            params={
                "kind": kind,
                "indirect": indirect,
                "guideline_id": guideline_id,
                "tag_id": tag_id,
                "tool_id": tool_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Relationship],
                    parse_obj_as(
                        type_=typing.List[Relationship],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        kind: RelationshipKindDto,
        source_guideline: typing.Optional[str] = OMIT,
        source_tag: typing.Optional[str] = OMIT,
        source_tool: typing.Optional[ToolId] = OMIT,
        target_guideline: typing.Optional[str] = OMIT,
        target_tag: typing.Optional[str] = OMIT,
        target_tool: typing.Optional[ToolId] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Relationship:
        """
        Create a relationship.

        A relationship is a relationship between a guideline and a tag.
        It can be created between a guideline and a tag, or between two guidelines, or between two tags.

        Parameters
        ----------
        kind : RelationshipKindDto

        source_guideline : typing.Optional[str]
            Unique identifier for the guideline

        source_tag : typing.Optional[str]
            Unique identifier for the tag

        source_tool : typing.Optional[ToolId]

        target_guideline : typing.Optional[str]
            Unique identifier for the guideline

        target_tag : typing.Optional[str]
            Unique identifier for the tag

        target_tool : typing.Optional[ToolId]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Relationship
            Relationship successfully created. Returns the created relationship.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.relationships.create(
                source_guideline="gid_123",
                target_tag="tid_456",
                kind="entailment",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "relationships",
            method="POST",
            json={
                "source_guideline": source_guideline,
                "source_tag": source_tag,
                "source_tool": convert_and_respect_annotation_metadata(
                    object_=source_tool, annotation=ToolId, direction="write"
                ),
                "target_guideline": target_guideline,
                "target_tag": target_tag,
                "target_tool": convert_and_respect_annotation_metadata(
                    object_=target_tool, annotation=ToolId, direction="write"
                ),
                "kind": kind,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Relationship,
                    parse_obj_as(
                        type_=Relationship,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve(
        self,
        relationship_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Relationship:
        """
        Read a relationship by ID.

        Parameters
        ----------
        relationship_id : str
            identifier of relationship

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Relationship
            Relationship successfully retrieved. Returns the requested relationship.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.relationships.retrieve(
                relationship_id="gr_123",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"relationships/{jsonable_encoder(relationship_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Relationship,
                    parse_obj_as(
                        type_=Relationship,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        relationship_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete a relationship by ID.

        Parameters
        ----------
        relationship_id : str
            identifier of relationship

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.relationships.delete(
                relationship_id="gr_123",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"relationships/{jsonable_encoder(relationship_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.guideline import Guideline
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.criticality_dto import CriticalityDto
from ..types.composition_mode_dto import CompositionModeDto
from ..types.guideline_with_relationships_and_tool_associations import (
    GuidelineWithRelationshipsAndToolAssociations,
)
from ..core.jsonable_encoder import jsonable_encoder
from ..errors.not_found_error import NotFoundError
from ..types.guideline_tool_association_update_params import (
    GuidelineToolAssociationUpdateParams,
)
from ..types.guideline_tags_update_params import GuidelineTagsUpdateParams
from ..types.guideline_metadata_update_params import GuidelineMetadataUpdateParams
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class GuidelinesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        tag_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Guideline]:
        """
        Lists all guidelines for the specified tag or all guidelines if no tag is provided.

        Returns an empty list if no guidelines exist.
        Guidelines are returned in no guaranteed order.
        Does not include relationships or tool associations.

        Parameters
        ----------
        tag_id : typing.Optional[str]
            The tag ID to filter guidelines by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Guideline]
            List of all guidelines for the specified tag or all guidelines if no tag is provided

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.guidelines.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "guidelines",
            method="GET",
            params={
                "tag_id": tag_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Guideline],
                    parse_obj_as(
                        type_=typing.List[Guideline],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        condition: str,
        id: typing.Optional[str] = OMIT,
        action: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        criticality: typing.Optional[CriticalityDto] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        composition_mode: typing.Optional[CompositionModeDto] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Guideline:
        """
        Creates a new guideline.

        The guideline will be initialized with the provided condition and optional action and settings.
        A unique identifier will be automatically generated unless a custom ID is provided.

        See the [documentation](https://parlant.io/docs/concepts/customization/guidelines) for more information.

        Parameters
        ----------
        condition : str
            If this condition is satisfied, the action will be performed

        id : typing.Optional[str]
            Unique identifier for the guideline

        action : typing.Optional[str]
            This action will be performed if the condition is satisfied

        description : typing.Optional[str]
            Optional description providing additional context for the guideline

        criticality : typing.Optional[CriticalityDto]

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Metadata for the guideline

        enabled : typing.Optional[bool]
            Whether the guideline is enabled

        tags : typing.Optional[typing.Sequence[str]]
            The tags associated with the guideline

        composition_mode : typing.Optional[CompositionModeDto]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Guideline
            Guideline successfully created. Returns the created guideline.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.guidelines.create(
            condition="when the customer asks about pricing",
            action="provide current pricing information and mention any ongoing promotions",
            metadata={"key1": "value1", "key2": "value2"},
            enabled=False,
            composition_mode="strict_canned",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "guidelines",
            method="POST",
            json={
                "id": id,
                "condition": condition,
                "action": action,
                "description": description,
                "criticality": criticality,
                "metadata": metadata,
                "enabled": enabled,
                "tags": tags,
                "composition_mode": composition_mode,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Guideline,
                    parse_obj_as(
                        type_=Guideline,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve(
        self,
        guideline_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GuidelineWithRelationshipsAndToolAssociations:
        """
        Retrieves a specific guideline with all its relationships and tool associations.

        Returns both direct and indirect relationships between guidelines.
        Tool associations indicate which tools the guideline can use.

        Parameters
        ----------
        guideline_id : str
            Unique identifier for the guideline

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GuidelineWithRelationshipsAndToolAssociations
            Guideline details successfully retrieved. Returns the complete guideline with its relationships and tool associations.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.guidelines.retrieve(
            guideline_id="IUCGT-l4pS",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"guidelines/{jsonable_encoder(guideline_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GuidelineWithRelationshipsAndToolAssociations,
                    parse_obj_as(
                        type_=GuidelineWithRelationshipsAndToolAssociations,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        guideline_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        guideline_id : str
            Unique identifier for the guideline

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.guidelines.delete(
            guideline_id="IUCGT-l4pS",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"guidelines/{jsonable_encoder(guideline_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        guideline_id: str,
        *,
        condition: typing.Optional[str] = OMIT,
        action: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        criticality: typing.Optional[CriticalityDto] = OMIT,
        tool_associations: typing.Optional[GuidelineToolAssociationUpdateParams] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        tags: typing.Optional[GuidelineTagsUpdateParams] = OMIT,
        metadata: typing.Optional[GuidelineMetadataUpdateParams] = OMIT,
        composition_mode: typing.Optional[CompositionModeDto] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GuidelineWithRelationshipsAndToolAssociations:
        """
        Updates a guideline's relationships and tool associations.

        Only provided attributes will be updated; others remain unchanged.

        Relationship rules:
        - A guideline cannot relate to itself
        - Only direct relationships can be removed
        - The relationship must specify this guideline as source or target

        Tool Association rules:
        - Tool services and tools must exist before creating associations

        Action with text can not be updated to None.

        Parameters
        ----------
        guideline_id : str
            Unique identifier for the guideline

        condition : typing.Optional[str]
            If this condition is satisfied, the action will be performed

        action : typing.Optional[str]
            This action will be performed if the condition is satisfied

        description : typing.Optional[str]
            Optional description providing additional context for the guideline

        criticality : typing.Optional[CriticalityDto]

        tool_associations : typing.Optional[GuidelineToolAssociationUpdateParams]

        enabled : typing.Optional[bool]
            Whether the guideline is enabled

        tags : typing.Optional[GuidelineTagsUpdateParams]

        metadata : typing.Optional[GuidelineMetadataUpdateParams]

        composition_mode : typing.Optional[CompositionModeDto]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GuidelineWithRelationshipsAndToolAssociations
            Guideline successfully updated. Returns the updated guideline with its relationships and tool associations.

        Examples
        --------
        from parlant.client import (
            GuidelineMetadataUpdateParams,
            GuidelineToolAssociationUpdateParams,
            ParlantClient,
            ToolId,
        )

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.guidelines.update(
            guideline_id="IUCGT-l4pS",
            condition="when the customer asks about pricing",
            action="provide current pricing information",
            tool_associations=GuidelineToolAssociationUpdateParams(
                add=[
                    ToolId(
                        service_name="new_service",
                        tool_name="new_tool",
                    )
                ],
                remove=[
                    ToolId(
                        service_name="old_service",
                        tool_name="old_tool",
                    )
                ],
            ),
            enabled=True,
            metadata=GuidelineMetadataUpdateParams(
                set_={"key1": "value1", "key2": "value2"},
                unset=["key3", "key4"],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"guidelines/{jsonable_encoder(guideline_id)}",
            method="PATCH",
            json={
                "condition": condition,
                "action": action,
                "description": description,
                "criticality": criticality,
                "tool_associations": convert_and_respect_annotation_metadata(
                    object_=tool_associations,
                    annotation=GuidelineToolAssociationUpdateParams,
                    direction="write",
                ),
                "enabled": enabled,
                "tags": convert_and_respect_annotation_metadata(
                    object_=tags,
                    annotation=GuidelineTagsUpdateParams,
                    direction="write",
                ),
                "metadata": convert_and_respect_annotation_metadata(
                    object_=metadata,
                    annotation=GuidelineMetadataUpdateParams,
                    direction="write",
                ),
                "composition_mode": composition_mode,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GuidelineWithRelationshipsAndToolAssociations,
                    parse_obj_as(
                        type_=GuidelineWithRelationshipsAndToolAssociations,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncGuidelinesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        tag_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Guideline]:
        """
        Lists all guidelines for the specified tag or all guidelines if no tag is provided.

        Returns an empty list if no guidelines exist.
        Guidelines are returned in no guaranteed order.
        Does not include relationships or tool associations.

        Parameters
        ----------
        tag_id : typing.Optional[str]
            The tag ID to filter guidelines by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Guideline]
            List of all guidelines for the specified tag or all guidelines if no tag is provided

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.guidelines.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guidelines",
            method="GET",
            params={
                "tag_id": tag_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Guideline],
                    parse_obj_as(
                        type_=typing.List[Guideline],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        condition: str,
        id: typing.Optional[str] = OMIT,
        action: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        criticality: typing.Optional[CriticalityDto] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        composition_mode: typing.Optional[CompositionModeDto] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Guideline:
        """
        Creates a new guideline.

        The guideline will be initialized with the provided condition and optional action and settings.
        A unique identifier will be automatically generated unless a custom ID is provided.

        See the [documentation](https://parlant.io/docs/concepts/customization/guidelines) for more information.

        Parameters
        ----------
        condition : str
            If this condition is satisfied, the action will be performed

        id : typing.Optional[str]
            Unique identifier for the guideline

        action : typing.Optional[str]
            This action will be performed if the condition is satisfied

        description : typing.Optional[str]
            Optional description providing additional context for the guideline

        criticality : typing.Optional[CriticalityDto]

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Metadata for the guideline

        enabled : typing.Optional[bool]
            Whether the guideline is enabled

        tags : typing.Optional[typing.Sequence[str]]
            The tags associated with the guideline

        composition_mode : typing.Optional[CompositionModeDto]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Guideline
            Guideline successfully created. Returns the created guideline.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.guidelines.create(
                condition="when the customer asks about pricing",
                action="provide current pricing information and mention any ongoing promotions",
                metadata={"key1": "value1", "key2": "value2"},
                enabled=False,
                composition_mode="strict_canned",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guidelines",
            method="POST",
            json={
                "id": id,
                "condition": condition,
                "action": action,
                "description": description,
                "criticality": criticality,
                "metadata": metadata,
                "enabled": enabled,
                "tags": tags,
                "composition_mode": composition_mode,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Guideline,
                    parse_obj_as(
                        type_=Guideline,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve(
        self,
        guideline_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GuidelineWithRelationshipsAndToolAssociations:
        """
        Retrieves a specific guideline with all its relationships and tool associations.

        Returns both direct and indirect relationships between guidelines.
        Tool associations indicate which tools the guideline can use.

        Parameters
        ----------
        guideline_id : str
            Unique identifier for the guideline

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GuidelineWithRelationshipsAndToolAssociations
            Guideline details successfully retrieved. Returns the complete guideline with its relationships and tool associations.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.guidelines.retrieve(
                guideline_id="IUCGT-l4pS",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"guidelines/{jsonable_encoder(guideline_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GuidelineWithRelationshipsAndToolAssociations,
                    parse_obj_as(
                        type_=GuidelineWithRelationshipsAndToolAssociations,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        guideline_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        guideline_id : str
            Unique identifier for the guideline

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.guidelines.delete(
                guideline_id="IUCGT-l4pS",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"guidelines/{jsonable_encoder(guideline_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        guideline_id: str,
        *,
        condition: typing.Optional[str] = OMIT,
        action: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        criticality: typing.Optional[CriticalityDto] = OMIT,
        tool_associations: typing.Optional[GuidelineToolAssociationUpdateParams] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        tags: typing.Optional[GuidelineTagsUpdateParams] = OMIT,
        metadata: typing.Optional[GuidelineMetadataUpdateParams] = OMIT,
        composition_mode: typing.Optional[CompositionModeDto] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GuidelineWithRelationshipsAndToolAssociations:
        """
        Updates a guideline's relationships and tool associations.

        Only provided attributes will be updated; others remain unchanged.

        Relationship rules:
        - A guideline cannot relate to itself
        - Only direct relationships can be removed
        - The relationship must specify this guideline as source or target

        Tool Association rules:
        - Tool services and tools must exist before creating associations

        Action with text can not be updated to None.

        Parameters
        ----------
        guideline_id : str
            Unique identifier for the guideline

        condition : typing.Optional[str]
            If this condition is satisfied, the action will be performed

        action : typing.Optional[str]
            This action will be performed if the condition is satisfied

        description : typing.Optional[str]
            Optional description providing additional context for the guideline

        criticality : typing.Optional[CriticalityDto]

        tool_associations : typing.Optional[GuidelineToolAssociationUpdateParams]

        enabled : typing.Optional[bool]
            Whether the guideline is enabled

        tags : typing.Optional[GuidelineTagsUpdateParams]

        metadata : typing.Optional[GuidelineMetadataUpdateParams]

        composition_mode : typing.Optional[CompositionModeDto]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GuidelineWithRelationshipsAndToolAssociations
            Guideline successfully updated. Returns the updated guideline with its relationships and tool associations.

        Examples
        --------
        import asyncio

        from parlant.client import (
            AsyncParlantClient,
            GuidelineMetadataUpdateParams,
            GuidelineToolAssociationUpdateParams,
            ToolId,
        )

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.guidelines.update(
                guideline_id="IUCGT-l4pS",
                condition="when the customer asks about pricing",
                action="provide current pricing information",
                tool_associations=GuidelineToolAssociationUpdateParams(
                    add=[
                        ToolId(
                            service_name="new_service",
                            tool_name="new_tool",
                        )
                    ],
                    remove=[
                        ToolId(
                            service_name="old_service",
                            tool_name="old_tool",
                        )
                    ],
                ),
                enabled=True,
                metadata=GuidelineMetadataUpdateParams(
                    set_={"key1": "value1", "key2": "value2"},
                    unset=["key3", "key4"],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"guidelines/{jsonable_encoder(guideline_id)}",
            method="PATCH",
            json={
                "condition": condition,
                "action": action,
                "description": description,
                "criticality": criticality,
                "tool_associations": convert_and_respect_annotation_metadata(
                    object_=tool_associations,
                    annotation=GuidelineToolAssociationUpdateParams,
                    direction="write",
                ),
                "enabled": enabled,
                "tags": convert_and_respect_annotation_metadata(
                    object_=tags,
                    annotation=GuidelineTagsUpdateParams,
                    direction="write",
                ),
                "metadata": convert_and_respect_annotation_metadata(
                    object_=metadata,
                    annotation=GuidelineMetadataUpdateParams,
                    direction="write",
                ),
                "composition_mode": composition_mode,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GuidelineWithRelationshipsAndToolAssociations,
                    parse_obj_as(
                        type_=GuidelineWithRelationshipsAndToolAssociations,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.canned_response import CannedResponse
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.canned_response_field import CannedResponseField
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.jsonable_encoder import jsonable_encoder
from ..errors.not_found_error import NotFoundError
from ..types.canned_response_tag_update_params import CannedResponseTagUpdateParams
from ..types.canned_response_metadata_update_params import (
    CannedResponseMetadataUpdateParams,
)
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CannedResponsesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        tags: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[CannedResponse]:
        """
        Lists all canned responses, optionally filtered by tags.

        Parameters
        ----------
        tags : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter canned responses by tags

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[CannedResponse]
            List of all canned responses in the system

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.canned_responses.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "canned_responses",
            method="GET",
            params={
                "tags": tags,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[CannedResponse],
                    parse_obj_as(
                        type_=typing.List[CannedResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        value: str,
        fields: typing.Sequence[CannedResponseField],
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        signals: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CannedResponse:
        """
        Parameters
        ----------
        value : str
            The textual content of the canned response.

        fields : typing.Sequence[CannedResponseField]
            A sequence of canned response fields associated with the canned response.

        tags : typing.Optional[typing.Sequence[str]]
            Collection of tag IDs associated with the canned response.

        signals : typing.Optional[typing.Sequence[str]]
            A sequence of signals associated with the canned response, to help with filtering and matching.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Additional metadata associated with the canned response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CannedResponse
            CannedResponse successfully created.

        Examples
        --------
        from parlant.client import CannedResponseField, ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.canned_responses.create(
            value="Your account balance is {balance}",
            fields=[
                CannedResponseField(
                    name="balance",
                    description="Account's balance",
                    examples=["9000"],
                )
            ],
            metadata={"category": "account", "priority": 1},
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "canned_responses",
            method="POST",
            json={
                "value": value,
                "fields": convert_and_respect_annotation_metadata(
                    object_=fields,
                    annotation=typing.Sequence[CannedResponseField],
                    direction="write",
                ),
                "tags": tags,
                "signals": signals,
                "metadata": metadata,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CannedResponse,
                    parse_obj_as(
                        type_=CannedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve(
        self,
        canned_response_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CannedResponse:
        """
        Retrieves details of a specific canned response by ID.

        Parameters
        ----------
        canned_response_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CannedResponse
            Canned response details successfully retrieved. Returns the CannedResponse object.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.canned_responses.retrieve(
            canned_response_id="t9a8g703f4",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"canned_responses/{jsonable_encoder(canned_response_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CannedResponse,
                    parse_obj_as(
                        type_=CannedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        canned_response_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        canned_response_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.canned_responses.delete(
            canned_response_id="t9a8g703f4",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"canned_responses/{jsonable_encoder(canned_response_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        canned_response_id: str,
        *,
        value: typing.Optional[str] = OMIT,
        fields: typing.Optional[typing.Sequence[CannedResponseField]] = OMIT,
        tags: typing.Optional[CannedResponseTagUpdateParams] = OMIT,
        metadata: typing.Optional[CannedResponseMetadataUpdateParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CannedResponse:
        """
        Updates an existing canned response's attributes.

        Only provided attributes will be updated; others remain unchanged.
        The canned response's ID and creation timestamp cannot be modified.
        Extra metadata and tags can be added or removed independently.

        Parameters
        ----------
        canned_response_id : str

        value : typing.Optional[str]
            The textual content of the canned response.

        fields : typing.Optional[typing.Sequence[CannedResponseField]]
            A sequence of canned response fields associated with the canned response.

        tags : typing.Optional[CannedResponseTagUpdateParams]

        metadata : typing.Optional[CannedResponseMetadataUpdateParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CannedResponse
            Canned response successfully updated. Returns the updated CannedResponse object.

        Examples
        --------
        from parlant.client import (
            CannedResponseField,
            CannedResponseMetadataUpdateParams,
            ParlantClient,
        )

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.canned_responses.update(
            canned_response_id="t9a8g703f4",
            value="Your updated balance is {balance}",
            fields=[
                CannedResponseField(
                    name="balance",
                    description="Updated account balance",
                    examples=["10000"],
                )
            ],
            metadata=CannedResponseMetadataUpdateParams(
                set_={"category": "account", "priority": 2},
                unset=["old_field"],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"canned_responses/{jsonable_encoder(canned_response_id)}",
            method="PATCH",
            json={
                "value": value,
                "fields": convert_and_respect_annotation_metadata(
                    object_=fields,
                    annotation=typing.Sequence[CannedResponseField],
                    direction="write",
                ),
                "tags": convert_and_respect_annotation_metadata(
                    object_=tags,
                    annotation=CannedResponseTagUpdateParams,
                    direction="write",
                ),
                "metadata": convert_and_respect_annotation_metadata(
                    object_=metadata,
                    annotation=CannedResponseMetadataUpdateParams,
                    direction="write",
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CannedResponse,
                    parse_obj_as(
                        type_=CannedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncCannedResponsesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        tags: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[CannedResponse]:
        """
        Lists all canned responses, optionally filtered by tags.

        Parameters
        ----------
        tags : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter canned responses by tags

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[CannedResponse]
            List of all canned responses in the system

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.canned_responses.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "canned_responses",
            method="GET",
            params={
                "tags": tags,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[CannedResponse],
                    parse_obj_as(
                        type_=typing.List[CannedResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        value: str,
        fields: typing.Sequence[CannedResponseField],
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        signals: typing.Optional[typing.Sequence[str]] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CannedResponse:
        """
        Parameters
        ----------
        value : str
            The textual content of the canned response.

        fields : typing.Sequence[CannedResponseField]
            A sequence of canned response fields associated with the canned response.

        tags : typing.Optional[typing.Sequence[str]]
            Collection of tag IDs associated with the canned response.

        signals : typing.Optional[typing.Sequence[str]]
            A sequence of signals associated with the canned response, to help with filtering and matching.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Additional metadata associated with the canned response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CannedResponse
            CannedResponse successfully created.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient, CannedResponseField

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.canned_responses.create(
                value="Your account balance is {balance}",
                fields=[
                    CannedResponseField(
                        name="balance",
                        description="Account's balance",
                        examples=["9000"],
                    )
                ],
                metadata={"category": "account", "priority": 1},
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "canned_responses",
            method="POST",
            json={
                "value": value,
                "fields": convert_and_respect_annotation_metadata(
                    object_=fields,
                    annotation=typing.Sequence[CannedResponseField],
                    direction="write",
                ),
                "tags": tags,
                "signals": signals,
                "metadata": metadata,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CannedResponse,
                    parse_obj_as(
                        type_=CannedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve(
        self,
        canned_response_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CannedResponse:
        """
        Retrieves details of a specific canned response by ID.

        Parameters
        ----------
        canned_response_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CannedResponse
            Canned response details successfully retrieved. Returns the CannedResponse object.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.canned_responses.retrieve(
                canned_response_id="t9a8g703f4",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"canned_responses/{jsonable_encoder(canned_response_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CannedResponse,
                    parse_obj_as(
                        type_=CannedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        canned_response_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        canned_response_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.canned_responses.delete(
                canned_response_id="t9a8g703f4",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"canned_responses/{jsonable_encoder(canned_response_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        canned_response_id: str,
        *,
        value: typing.Optional[str] = OMIT,
        fields: typing.Optional[typing.Sequence[CannedResponseField]] = OMIT,
        tags: typing.Optional[CannedResponseTagUpdateParams] = OMIT,
        metadata: typing.Optional[CannedResponseMetadataUpdateParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CannedResponse:
        """
        Updates an existing canned response's attributes.

        Only provided attributes will be updated; others remain unchanged.
        The canned response's ID and creation timestamp cannot be modified.
        Extra metadata and tags can be added or removed independently.

        Parameters
        ----------
        canned_response_id : str

        value : typing.Optional[str]
            The textual content of the canned response.

        fields : typing.Optional[typing.Sequence[CannedResponseField]]
            A sequence of canned response fields associated with the canned response.

        tags : typing.Optional[CannedResponseTagUpdateParams]

        metadata : typing.Optional[CannedResponseMetadataUpdateParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CannedResponse
            Canned response successfully updated. Returns the updated CannedResponse object.

        Examples
        --------
        import asyncio

        from parlant.client import (
            AsyncParlantClient,
            CannedResponseField,
            CannedResponseMetadataUpdateParams,
        )

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.canned_responses.update(
                canned_response_id="t9a8g703f4",
                value="Your updated balance is {balance}",
                fields=[
                    CannedResponseField(
                        name="balance",
                        description="Updated account balance",
                        examples=["10000"],
                    )
                ],
                metadata=CannedResponseMetadataUpdateParams(
                    set_={"category": "account", "priority": 2},
                    unset=["old_field"],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"canned_responses/{jsonable_encoder(canned_response_id)}",
            method="PATCH",
            json={
                "value": value,
                "fields": convert_and_respect_annotation_metadata(
                    object_=fields,
                    annotation=typing.Sequence[CannedResponseField],
                    direction="write",
                ),
                "tags": convert_and_respect_annotation_metadata(
                    object_=tags,
                    annotation=CannedResponseTagUpdateParams,
                    direction="write",
                ),
                "metadata": convert_and_respect_annotation_metadata(
                    object_=metadata,
                    annotation=CannedResponseMetadataUpdateParams,
                    direction="write",
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CannedResponse,
                    parse_obj_as(
                        type_=CannedResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.journey import Journey
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.composition_mode_dto import CompositionModeDto
from ..core.jsonable_encoder import jsonable_encoder
from ..errors.not_found_error import NotFoundError
from ..types.journey_condition_update_params import JourneyConditionUpdateParams
from ..types.journey_tag_update_params import JourneyTagUpdateParams
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class JourneysClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        tag_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Journey]:
        """
        Retrieves a list of all journeys in the system.

        Parameters
        ----------
        tag_id : typing.Optional[str]
            The tag ID to filter journeys by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Journey]
            List of all journeys in the system

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.journeys.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "journeys",
            method="GET",
            params={
                "tag_id": tag_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Journey],
                    parse_obj_as(
                        type_=typing.List[Journey],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        title: str,
        description: str,
        conditions: typing.Sequence[str],
        id: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        composition_mode: typing.Optional[CompositionModeDto] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Journey:
        """
        Creates a new journey in the system.

        The journey will be initialized with the provided title, description, and conditions.
        A unique identifier will be automatically generated unless a custom ID is provided.

        Parameters
        ----------
        title : str
            The title of the journey

        description : str

        conditions : typing.Sequence[str]

        id : typing.Optional[str]
            Unique identifier for the journey

        tags : typing.Optional[typing.Sequence[str]]
            List of tag IDs associated with the journey

        composition_mode : typing.Optional[CompositionModeDto]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Journey
            Journey successfully created. Returns the complete journey object including generated ID.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.journeys.create(
            title="Customer Onboarding",
            description="1. Customer wants to lock their card\n2. Customer reports that their card doesn't work\n3. Customer suspects their card has been stolen",
            conditions=[
                "customer needs unlocking their card",
                "customer needs help with card",
            ],
            id="IUCGT-lvpS",
            tags=["tag1", "tag2"],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "journeys",
            method="POST",
            json={
                "title": title,
                "description": description,
                "conditions": conditions,
                "id": id,
                "tags": tags,
                "composition_mode": composition_mode,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Journey,
                    parse_obj_as(
                        type_=Journey,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve(
        self,
        journey_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Journey:
        """
        Retrieves details of a specific journey by ID.

        Parameters
        ----------
        journey_id : str
            Unique identifier for the journey

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Journey
            Journey details successfully retrieved. Returns the complete journey object.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.journeys.retrieve(
            journey_id="IUCGT-lvpS",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"journeys/{jsonable_encoder(journey_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Journey,
                    parse_obj_as(
                        type_=Journey,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        journey_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a journey from the system.

        Also deletes the associated guideline.
        Deleting a non-existent journey will return 404.
        No content will be returned from a successful deletion.

        Parameters
        ----------
        journey_id : str
            Unique identifier for the journey

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.journeys.delete(
            journey_id="IUCGT-lvpS",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"journeys/{jsonable_encoder(journey_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        journey_id: str,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        conditions: typing.Optional[JourneyConditionUpdateParams] = OMIT,
        tags: typing.Optional[JourneyTagUpdateParams] = OMIT,
        composition_mode: typing.Optional[CompositionModeDto] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Journey:
        """
        Updates an existing journey's attributes.

        Only the provided attributes will be updated; others will remain unchanged.

        Parameters
        ----------
        journey_id : str
            Unique identifier for the journey

        title : typing.Optional[str]
            The title of the journey

        description : typing.Optional[str]

        conditions : typing.Optional[JourneyConditionUpdateParams]

        tags : typing.Optional[JourneyTagUpdateParams]

        composition_mode : typing.Optional[CompositionModeDto]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Journey
            Journey successfully updated. Returns the updated journey.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.journeys.update(
            journey_id="IUCGT-lvpS",
            title="Customer Onboarding",
            description="1. Customer wants to lock their card\n2. Customer reports that their card doesn't work\n3. Customer suspects their card has been stolen",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"journeys/{jsonable_encoder(journey_id)}",
            method="PATCH",
            json={
                "title": title,
                "description": description,
                "conditions": convert_and_respect_annotation_metadata(
                    object_=conditions,
                    annotation=JourneyConditionUpdateParams,
                    direction="write",
                ),
                "tags": convert_and_respect_annotation_metadata(
                    object_=tags, annotation=JourneyTagUpdateParams, direction="write"
                ),
                "composition_mode": composition_mode,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Journey,
                    parse_obj_as(
                        type_=Journey,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def mermaid(
        self,
        journey_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> str:
        """
        Returns the journey as a Mermaid 'stateDiagramv-v2' string.
        Content-Type: text/plain

        Parameters
        ----------
        journey_id : str
            Unique identifier for the journey

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            Mermaid stateDiagram V2 (text/plain). Copy/paste directly into a Mermaid renderer.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.journeys.mermaid(
            journey_id="journey_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"journeys/{jsonable_encoder(journey_id)}/mermaid",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return _response.text  # type: ignore
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncJourneysClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        tag_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Journey]:
        """
        Retrieves a list of all journeys in the system.

        Parameters
        ----------
        tag_id : typing.Optional[str]
            The tag ID to filter journeys by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Journey]
            List of all journeys in the system

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.journeys.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "journeys",
            method="GET",
            params={
                "tag_id": tag_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Journey],
                    parse_obj_as(
                        type_=typing.List[Journey],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        title: str,
        description: str,
        conditions: typing.Sequence[str],
        id: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        composition_mode: typing.Optional[CompositionModeDto] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Journey:
        """
        Creates a new journey in the system.

        The journey will be initialized with the provided title, description, and conditions.
        A unique identifier will be automatically generated unless a custom ID is provided.

        Parameters
        ----------
        title : str
            The title of the journey

        description : str

        conditions : typing.Sequence[str]

        id : typing.Optional[str]
            Unique identifier for the journey

        tags : typing.Optional[typing.Sequence[str]]
            List of tag IDs associated with the journey

        composition_mode : typing.Optional[CompositionModeDto]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Journey
            Journey successfully created. Returns the complete journey object including generated ID.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.journeys.create(
                title="Customer Onboarding",
                description="1. Customer wants to lock their card\n2. Customer reports that their card doesn't work\n3. Customer suspects their card has been stolen",
                conditions=[
                    "customer needs unlocking their card",
                    "customer needs help with card",
                ],
                id="IUCGT-lvpS",
                tags=["tag1", "tag2"],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "journeys",
            method="POST",
            json={
                "title": title,
                "description": description,
                "conditions": conditions,
                "id": id,
                "tags": tags,
                "composition_mode": composition_mode,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Journey,
                    parse_obj_as(
                        type_=Journey,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve(
        self,
        journey_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Journey:
        """
        Retrieves details of a specific journey by ID.

        Parameters
        ----------
        journey_id : str
            Unique identifier for the journey

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Journey
            Journey details successfully retrieved. Returns the complete journey object.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.journeys.retrieve(
                journey_id="IUCGT-lvpS",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"journeys/{jsonable_encoder(journey_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Journey,
                    parse_obj_as(
                        type_=Journey,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        journey_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a journey from the system.

        Also deletes the associated guideline.
        Deleting a non-existent journey will return 404.
        No content will be returned from a successful deletion.

        Parameters
        ----------
        journey_id : str
            Unique identifier for the journey

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.journeys.delete(
                journey_id="IUCGT-lvpS",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"journeys/{jsonable_encoder(journey_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        journey_id: str,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        conditions: typing.Optional[JourneyConditionUpdateParams] = OMIT,
        tags: typing.Optional[JourneyTagUpdateParams] = OMIT,
        composition_mode: typing.Optional[CompositionModeDto] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Journey:
        """
        Updates an existing journey's attributes.

        Only the provided attributes will be updated; others will remain unchanged.

        Parameters
        ----------
        journey_id : str
            Unique identifier for the journey

        title : typing.Optional[str]
            The title of the journey

        description : typing.Optional[str]

        conditions : typing.Optional[JourneyConditionUpdateParams]

        tags : typing.Optional[JourneyTagUpdateParams]

        composition_mode : typing.Optional[CompositionModeDto]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Journey
            Journey successfully updated. Returns the updated journey.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.journeys.update(
                journey_id="IUCGT-lvpS",
                title="Customer Onboarding",
                description="1. Customer wants to lock their card\n2. Customer reports that their card doesn't work\n3. Customer suspects their card has been stolen",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"journeys/{jsonable_encoder(journey_id)}",
            method="PATCH",
            json={
                "title": title,
                "description": description,
                "conditions": convert_and_respect_annotation_metadata(
                    object_=conditions,
                    annotation=JourneyConditionUpdateParams,
                    direction="write",
                ),
                "tags": convert_and_respect_annotation_metadata(
                    object_=tags, annotation=JourneyTagUpdateParams, direction="write"
                ),
                "composition_mode": composition_mode,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Journey,
                    parse_obj_as(
                        type_=Journey,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def mermaid(
        self,
        journey_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> str:
        """
        Returns the journey as a Mermaid 'stateDiagramv-v2' string.
        Content-Type: text/plain

        Parameters
        ----------
        journey_id : str
            Unique identifier for the journey

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            Mermaid stateDiagram V2 (text/plain). Copy/paste directly into a Mermaid renderer.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.journeys.mermaid(
                journey_id="journey_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"journeys/{jsonable_encoder(journey_id)}/mermaid",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return _response.text  # type: ignore
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import datetime as dt
import typing
from .canned_response_field import CannedResponseField
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CannedResponse(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    Unique identifier for the tag
    """

    creation_utc: dt.datetime = pydantic.Field()
    """
    UTC timestamp of when the canned response was created
    """

    value: str = pydantic.Field()
    """
    The textual content of the canned response.
    """

    fields: typing.List[CannedResponseField] = pydantic.Field()
    """
    A sequence of canned response fields associated with the canned response.
    """

    tags: typing.List[str] = pydantic.Field()
    """
    Collection of tag IDs associated with the canned response.
    """

    signals: typing.List[str] = pydantic.Field()
    """
    A sequence of signals associated with the canned response, to help with filtering and matching.
    """

    metadata: typing.Dict[str, typing.Optional[typing.Any]] = pydantic.Field()
    """
    Additional metadata associated with the canned response.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
